/* sound.c : for xlevel 
 * programmed by K. Takebe, 1997/11/02
 * Referenced:  Linux Multimedia Guide (Jeff Tranter; O'Reilly)
 */
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#if   defined(linux)
#include <sys/soundcard.h>
#elif defined(__FreeBSD__)
#include <machine/soundcard.h>
#endif

#include "xlevel.h"

int  sampling = 44000;

unsigned char buffer[NDAT_MAX*2+2];     /* buffer for sound data */
static int snd = -1;                  /* file descriptor for sound device */
static int dma = DMA_DIV;             /* DMA buffer divisor */
static int actual;                    /* actual sampling rate */


int close_sound_device()
{
  if ( snd != -1 )
    close(snd);
  snd = -1;
  return 0;
}

/* initialize sampling data to zero level */
void init_sampling_buffer()
{
  int i;

  for (i = 0 ; i < NDAT_MAX*2 ; i++)
    buffer[i] = 128;
}


/* initialize /dev/dsp */
void init_sound_card(char *dsp_device)
{
  int parm;
  int status;

  if ( dsp_device == NULL ) {
    fprintf(stderr, "init_sound_card(): device name is NULL string. ");
    exit(1);
  }  

  /* open DSP device for read */
  snd = open(dsp_device, O_RDONLY);
  if (snd < 0) {
    fprintf(stderr, "init_sound_card(): cannot open %s. ", dsp_device);
    perror("");
    exit(1);
  }
  
  /* set stereo */
  parm = 2;
  status = ioctl(snd, SOUND_PCM_WRITE_CHANNELS, &parm);
  if (status < 0) {
    perror("init_sound_card(): error from sound device ioctl");
    exit(1);
  }

  /* set 8-bit samples */
  parm = 8;
  status = ioctl(snd, SOUND_PCM_WRITE_BITS, &parm);
  if (status < 0) {
    perror("init_sound_card(): error from sound device ioctl");
    exit(1);
  }
  
  /* set DMA buffer size */
  status = ioctl(snd, SOUND_PCM_SUBDIVIDE, &dma);
  if (status < 0) {
    perror("init_sound_card(): error from sound device ioctl");
    exit(1);
  }
  
  /* set sampling rate */
  parm = sampling;
  status = ioctl(snd, SOUND_PCM_WRITE_RATE, &parm);
  if (status < 0) {
    perror("init_sound_card(): error from sound device ioctl");
  }
  ioctl(snd, SOUND_PCM_READ_RATE, &actual);
  sampling = actual;

  if (debug || verbose) {
    printf("\taudio device file  = %s\n", dsp_device);
    printf("\tsampling_frequency = %d Hz\n", sampling);
  }
}


/* get data from sound card */
void get_data()
{
  int status;

  /* now get the real data */  
  status = read(snd, buffer, nDat*2);
  if (status == 0) {
    fprintf(stderr, "get_data(): end of file\n");
    close(snd);
    exit(1);
  }
  if (status == 1) {
    perror("get_data(): error reading input");
    close(snd);
    exit(1);
  }
}
